function params = coordsToDH4(atomTypeNo, chain)
% params = coordsToDH4(atomTypeNo, chain) compute DH4 parameters 
% (dihedral, bond ang, length offset)
% for the given chain(s), so that the dihedrals are phi/psi angles.
% This is not a reversable transformation, because information is lost on
% the exact placement of peptide atoms. 
% 
% chain starts with C or N, depending on whether first dihedral is phi or psi.
% Then between Cas it calculates P1 and P2, which are the two closest points 
% on lines Ca1-C and Ca2-N, which determine the offset distance
% 

count = 1;
kk = size(chain,3); mm = size(chain,4);
params = zeros(size(chain,1)-2, 4, kk, mm);

vec = diff(chain); % compute all vectors and normalize
len = sqrt(sum(vec.^2,2)); % get all lengths
vec = vec ./ len(:,[1 1 1], :, :); % normalize

if (atomTypeNo(1) == 3) % start with C
    params(count,2,
    
%first we compute new points that give offsets
ppChain = chain; % for help in computing offsets
if atomTypeNo(1) == 1
Cas = find(atomTypeNo(1:end-3) == 2); % list indices of all Calphas except last one.
for k = 1:size(chain,3)
    for count = 1:length(Cas) %iterate over all Calphas except last.
        Ca = Cas(count);
        % we find the two closest points P1 and P2 on the lines Ca1-C and Ca2-N
        vec1 = chain(Ca+1,:,k)-chain(Ca,:,k);   % Calpha1 - C
        vec2 = chain(Ca+2,:,k)-chain(Ca+3,:,k); % Calpha2 - N
        vec1 = vec1 / norm(vec1); 
        vec2 = vec2 / norm(vec2);      
        vec3 = chain(Ca,:,k)  -chain(Ca+3,:,k);   % Calpha2 - Calpha1
        
        dot12 = vec1'*vec2; 
        A = [-1     dot12;   
            -dot12   1   ];
        B = [dot(vec1,vec3); dot(vec2,vec3)]; %%% A*t = B;
        t = A\B; % these are the distances of P1 and P2 along vec1 & vec2
        
        P1 = chain(Ca,:,k)   + t(1)*vec1; 
        P2 = chain(Ca+3,:,k) + t(2)*vec2;
        
        vecP = P2-P1; %double check that they are closest
        if (dot(vec1,vecP)> 1e-9) 
            error('p1,p2 calculation not accurate'); end;
        if (dot(vec2,vecP)> 1e-9) 
            error('p1,p2 calculation not accurate'); end;
        
        ppChain(Ca+1:Ca+2,:,k) = [P1; P2]; % Replace N and C in ppChain
    end
end



numAtoms = length(chain);
deg = 180/pi;
numBonds = numAtoms-1;
bond = zeros(numBonds,3);
bondLengths = zeros(numBonds,1);

for i = 1:(numBonds)
    bond(i,:) = chain(i+1,:)-chain(i,:);
    bondLengths(i) = norm(bond(i,:));
    bond(i,:) = bond(i,:)./bondLengths(i);
end;

% record offset parameter (a) and remove P1->P2 info from bond info
aInd = firstN;
firstCa = 1+mod(firstN,3);
a = zeros(6,1);
offsetVec = zeros(8,3);
ikChain = chain;
ikBond = bond;

a(firstN:2:5) = bondLengths(firstCa+1:3:numBonds-2);
bondLengths(firstCa+1:3:numBonds-2) = [];
ikChain(firstCa+1:3:numBonds-2,:) = [];
ikBond(firstCa+1:3:numBonds-2,:) = [];
offsetVec(firstCa:2:6,:) = bond(firstCa+1:3:numBonds-2,:);
offsetVec(firstCa+1:2:7,:) = bond(firstCa+1:3:numBonds-2,:);

% 11 atoms -> was 10 bonds -> removed 2 -> 8 bonds -> 6 full params to come
% 12 atoms -> was 11 bonds -> removed 3 -> 8 bonds -> 6 full params to come

numIkBonds = length(ikBond);
bondAngles = zeros(numIkBonds,1);
dih = zeros(numIkBonds,1);

bondAngles(1) = acos(dot(ikBond(2,:),-ikBond(1,:)))*deg;  % convert to degrees
for i = 2:(numIkBonds-1)
    bondAngles(i) = acos(dot(ikBond(i+1,:),-ikBond(i,:)))*deg;  % convert to degrees
    bondAngles(i) = bondAngles(i)+180;     
    
    y = cross(ikBond(i-1,:),ikBond(i,:));
    y = y/norm(y);
    x = cross(y,ikBond(i,:));
    x = x/norm(x);
    x = dot(ikBond(i+1,:),x);
    y = dot(ikBond(i+1,:),y);
    dih(i) = dihedral(u,v,w)*deg;
    atan2(y,x)*deg; % convert to degrees
    
    crossCheck = cross(-ikBond(i,:),ikBond(i+1,:)); % x vector
    if dot(crossCheck, offsetVec(i,:)) < 0
        a(i-1) = -a(i-1);
    end;
end;

geom = zeros(numIkBonds-2,4);
geom(:,1) = dih(2:numIkBonds-1);
geom(:,2) = bondAngles(2:numIkBonds-1);
geom(:,3) = bondLengths(2:numIkBonds-1);
geom(:,4) = a;
        