function result = iksolnLoop(loop)
% produces ikParams [dihedral, bondDHangle, length, offset] for r6solver.
% 
% We replace all the C & N between Calphas with P1 & P2 to find offsets.
% Then we drop the P1s. They can be recovered by taking the position before
% applying the offset.  We keep cnPos multipliers that allow
% us to recover the replaced C & N positions.
% The ikParams are set so their dihedrals have rotational axes of phis & psis
% (although the angles may be off when there are non-zero offsets).
% bondDHangles are like bond angles -pi, so they are in [-pi,0].
% We make the targetFrame so the startFrame can be eye(4).

chain = loop.coords;
kk = size(chain,3);
% First, we replace C and N with P1 and P2.
[ppChain, cnPos] = peptideCNtoP1P2(loop.atomTypeNo, loop.coords);

c = peptideP1P2toCN(loop.atomTypeNo, ppChain, cnPos); % audit
if norm(loop.coords(:) - c(:)) > 1e-9, 
    loop.coords - c,
    fprintf('error in peptide conversion\n'); 
end

% Make start & target frames while we still have P1s
startFrame = makeFrame(ppChain(1:3,:, loop.original)); % either cb:C N Ca or 2cb:N Ca P1. Dihedral is a phi or psi. 
targetFrame = startFrame \ makeFrame(ppChain(end-2:end,:, loop.original)); % target if we start with eye(4)

% create the ikParams whose dihedrals are on phi/psi axes. 
ikParams = makeIkParams(loop.atomTypeNo, ppChain);
pp = recoverPPchain(ikParams, loop.atomTypeNo, ppChain, startFrame); %audit
if norm(ppChain(:) - pp(:)) > 1e-9, 
    ppChain - pp, 
    fprintf('error in ppChain-ikParam conversion\n'); 
end

MAXSOLN = 4; % will grow as needed.
kk = size(loop.coords,3);
solnCount = zeros(kk,1); % number of good solutions
solnD = zeros(MAXSOLN,1); % solution distance from original
coords = zeros(size(ppChain,1), 3, kk, MAXSOLN); % make space for solution. 
ppOrig = ppChain(:,:,loop.original);
for k = 1:kk
    % get Ik Geom = 6x4 = [dih ang, bond ang, bond len, offset]
    ikSolution = r6solverdeg(ikParams(:,[4 3 2],k), targetFrame);
    mm = size(ikSolution,2);
    solnIkParams = ikParams(:,:,k);
    for m = 1:mm
        solnIkParams(:,1) = ikSolution(:,m); % replace dihedrals with solution.
        solnChain = recoverPPchain(solnIkParams, loop.atomTypeNo, ppOrig, startFrame);
        d = norm(ppOrig(end-2:end,:) - solnChain(end-2:end,:)); 
        if (d < 1e-4) % hits target
            solnCount(k) = solnCount(k)+1;
            coords(:,:,k,solnCount(k)) = peptideP1P2toCN(loop.atomTypeNo, solnChain, cnPos(:,:,k));
            solnD(solnCount(k)) = norm(ppOrig(:) - solnChain(:));
        else
            %fprintf('discarding solution %d,%d with norm %g\n',k,m,d)
        end
    end
    if solnCount(k) == 0
        fprintf('No solutions for %d\n',k);
    else
        [solnD(1:solnCount(k)) ind] = sort(solnD(1:solnCount(k))); % put in order of increasing distance from original
        coords(:,:,k,1:length(ind)) = coords(:,:,k,ind);
    end
end

ind = solnCount>0; % remove configs with no soln

result.atomTypeNo = loop.atomTypeNo;
result.solnCount = solnCount(ind);
result.coords = coords(:,:,ind,:);
result.sidecoords = makeSideCoords(result);
result.startAtom = loop.startAtom;
result.wigName = loop.wigName;
result.wigDelta = loop.wigDelta(ind);
result.wigIdeal = loop.wigIdeal;
result.wigPosition = loop.wigPosition;
ind = cumsum(ind); % new indices corresponding to old ks
result.original = ind(loop.original);
