function ikParams = makeIkParams(atomTypeNo, chain)
% ikParams= makeIkParams(atomTypeNo, chain) takes ppChain and
% produces ikParams [dihedral, bondDHangle, length, offset] for r6solver.
% Thus, 6 dof is hardcoded; this will need to change when we generalize. 
%
% The ikParams are set so their dihedrals have rotational axes of phis & psis
% (although the angles may be off when there are non-zero offsets).
% bondDHangles are like bond angles -pi, so they are in [-pi,0].

kk = size(chain,3);
Cas = find(atomTypeNo == 2); % indices of all Calphas.
firstCa = min(Cas); lastCa = max(Cas); % We drop P1s between the Calphas
P1s = firstCa+1:3:lastCa-2; % locations of P1s to discard

if ~((firstCa==2 && length(atomTypeNo)==12)||(firstCa==3 && length(atomTypeNo)==11))
    fprintf('Unexpected case in makeIkParams: firstCa = %d, length = %d\n', firstCa, length(atomTypeNo));
end

ndof = length(atomTypeNo)-length(P1s)-3;
if (ndof ~= 6)
    fprintf('expected 6 dof, got %d\n', ndof);
end
ikParams = zeros(ndof, 4, kk);

bond = diff(chain); % compute all vectors and normalize
bondLen = sqrt(sum(bond.^2,2)); % get all lengths
bond = bond(:,:,:) ./ bondLen(:, [1 1 1], :); % normalize bonds

for k = 1:kk % fix signs for offsets
    for i = P1s
        if det(bond(i-1:i+1,:,k)) < 0 % makes [x:=p2-p1; y:=ca-p1; z:=p2-c1] right handed coords
            bondLen(i,k) = - bondLen(i,k);
        end
    end
end

ikParams(firstCa-1:2:ndof-1, 4, :) = bondLen(P1s,:); % non-zero offsets a with Calphas (sign TBD)
bond(P1s,:,:) = []; % discard P1s
bondLen(P1s,:) = [];
ikParams(:,3,:) = bondLen(2:end-1,:); % kept bond lengths as d

deg = 180/pi;
for k = 1:kk                      
    for i = 1:ndof
        ikParams(i,1,k) = computeDihedral(bond(i:i+2,:,k))*deg; % convert to degrees
        ikParams(i,2,k) = computeBondDHAngle(bond(i+1:i+2,:,k))*deg;  % convert to degrees    
    end
end

c = recoverPPchain(ikParams, atomTypeNo, chain, makeFrame(chain(1:3,:,1))); % audit
if norm(c(:) - chain(:)) > 1e-9
    error('Cannot recover ppChain from ikParams');
end