function result = wedemeyer(chain)
% An implementation of Wedemeyer and Scheraga's loop closure: Jack Snoeyink  14 July 03
% Function result = wedemeyer(chain) takes the coordinates (9x3) for three peptides
% n1, ca1, ... ca3, c3, and solves for all positions that respect bond lengths, 
% bond angles, and peptide planarity.  It returns their coordinates in a 9x3xk matrix.
%
% If your peptides are not planar, it will complain and probably give garbage results. 
% The function planarize can be used to make peptide bonds planar.  
%
% I think it works, up to numerical precision issues.  
% (I can take the solutions I get for a configuration, plug them in, 
%  and get the same set of configurations again...)
% It prints a few messages so you can judge the quality of solutions, 
% and has many asserts to check internal invariants, 
% so let snoeyink@cs.unc.edu know of instances where you see ASSERT messages.   


% atom i coordinates are in chain(i,:)
n1 = 1; ca1 = 2; c1 = 3;
n2 = 4; ca2 = 5; c2 = 6;
n3 = 7; ca3 = 8; c3 = 9;

%v contains p1 p2 p3, q1 q3, r, s1 s2 s3, in paper's notation. + peptide bond atoms
p1=1; p2=2; p3=3; q1=4; q3=5; r=6; s1=7; s2=8; s3=9; pb1=10; pb3=11;
v = chain([ca1 ca2 n3, ca2  ca2, ca3, c1  ca2 ca3, n2 c2], :) ...
   -chain([n1  n2 ca3, ca1  ca3, ca1, ca1 c2  c3,  ca1 ca3], :);

lengths = sqrt(dot(v,v,2));
%make unit vectors
V = v./repmat(lengths,1,3); 

%ang contains three of each fixed angle alpha,beta,gamma,delta 
%  and initial value of variable angles lambda1 & lambda3
ang = acos(dot(V([p1 p2 s3, q1 q1 q3, q1 q3 p3, p1 p2 p3, q1 q3],:), ...
               V([r  q1 r,  r  q3 r,  s1 s2 q3, s1 s2 s3, p1 s3],:),2));
A1=ang(1); A2=ang(2); A3=ang(3); B1=ang(4); B2=ang(5); B3=ang(6); 
G1=ang(7); G2=ang(8); G3=ang(9); D1=ang(10); D2=ang(11); D3=ang(12); 
L1=ang(13); L3=ang(14);
degrees = pi/180.0;
%initAngles = ang/degrees

assert(max(abs( [A2 G3 G1 G2]/degrees - [14.9 14.9 20.9 20.9])) < 3, ...
    'unexpected alpha or gamma',[A2 G3 G1 G2]/degrees);

% spherical angles by cross products 
om0=1; om1=2; om3=3;  ze1=4; ze3=5;  zeA=6; zeB=7;  nu1=8; % plus other test angles
sa = dihedral(V([s3 q1 q1, s1 p3, p1 s1 q3, p2 s2, q1 s1 s2], :), ...
              V([r  r  r,  q1 q3, q1 q1 q1, q1 q3, r  q1 q3], :), ...
              V([p1 p1 s3, r  r,  r  p1 p2, r  q1, q3 p2 p3], :));
          
%[sa sin(sa) cos(sa) sa/degrees]

% do some checks
assert(all(abs(sa(11:13))<1e-8),'non-planar peptides', [sa(11:13) sa(11:13)/degrees]);
assert(abs(sa(om3)+sa(om0) - sa(om1))< 1e-6, 'om0-om1+om3~=0', [sa(om0) sa(om1) sa(om3)]);
assert(abs(sa(zeA)+sa(zeB) - sa(ze1))< 1e-6, 'ze1 ~= zeA+zeB', [sa(ze1) sa(zeA) sa(zeB)]);
assert(abs(abs(sa(ze1)+sa(nu1)) - pi)< 1e-6, '|zeta1+nu1| ~= pi', [sa(ze1) sa(nu1) abs(sa(ze1)+sa(nu1))]);

w1 = tan(sa(om1)/2); % input versions of these angle tangents should be among the solutions we find
z1 = tan(sa(ze1)/2);
z3 = tan(sa(ze3)/2);

assert(abs(-cos(sa(ze1)) + (1-z1^2)/(1+z1^2))<1e-8, 'cos ze1 wrong', [cos(sa(ze1))  (1-z1^2)/(1+z1^2)]);
assert(abs(-sin(sa(ze1)) + (2*z1)/(1+z1^2))<1e-8, 'sin ze1 wrong', [sin(sa(ze1))  (2*z1)/(1+z1^2)]);
assert(abs(-cos(sa(om1)) + (1-w1^2)/(1+w1^2))<1e-8, 'cos om1 wrong', [cos(sa(om1))  (1-w1^2)/(1+w1^2)]);
assert(abs(-sin(sa(om1)) + (2*w1)/(1+w1^2))<1e-8, 'sin om1 wrong', [sin(sa(om1))  (2*w1)/(1+w1^2)]);


% first biquadratic coefficients: eqn (1)
CC = [cos(A1)*cos(B1)*cos(G1)-cos(D1)
      sin(A1)*sin(B1)*cos(G1)
      cos(A1)*sin(B1)*sin(G1)
     -sin(A1)*cos(B1)*sin(G1)
      sin(A1)*sin(G1)];

DD = [1 1 1 1 0; 1 -1 1 -1 0; 1 1 -1 -1 0; 1 -1 -1 1 0; 0 0 0 0 4] * CC;

assert(abs([1 w1^2 z1^2 w1^2*z1^2 w1*z1]*DD) < 1e-9, 'DD polynomial (1) non-zero', ...
    [1 w1^2 z1^2 w1^2*z1^2 w1*z1]*DD);

% third biquadratic coefficients: eqn (6)
LL = [cos(A2)*cos(B2)*cos(G2)-cos(D2)
      sin(A2)*sin(B2)*cos(G2)
      cos(A2)*sin(B2)*sin(G2)
     -sin(A2)*cos(B2)*sin(G2)
      sin(A2)*sin(G2)];

MM = [1 -1 1 -1 0; 1 1 1 1 0; 1 -1 -1 1 0; 1 1 -1 -1 0; 0 0 0 0 4] * LL;

assert(abs([1 z1^2 z3^2 z1^2*z3^2 z1*z3]*MM) < 1e-9, 'MM polynomial (6) non-zero', ...
    [1 z1^2 z3^2 z1^2*z3^2 z1*z3]*MM);

% second biquadratic coefficients: eqn (4)
NN = [cos(A3)*cos(B3)*cos(G3)-cos(D3)
      sin(A3)*sin(B3)*cos(G3)
      cos(A3)*sin(B3)*sin(G3)
     -sin(A3)*cos(B3)*sin(G3)
      sin(A3)*sin(G3)];

so = sin(sa(om0)); % need omega0 angle for this one.
co = cos(sa(om0));
%disp([dot(pr(p1r,:),pr(s3r,:)), co, sqrt(1-so^2)]); % check: should be equal.

PP = [1  co  1  co 0; 0 2*so 0  2*so 0; 0 0 0 0 -2*so; 0 0 0 0 4*co; % p00 10 01 11
      1 -co  1 -co 0;                                                % p20
      1  co -1 -co 0; 0 2*so 0 -2*so 0; 0 0 0 0  2*so;               % p02 12 21 
      1 -co -1  co 0]*NN;                                            % p22

coeff = [1 w1 z3 w1*z3 w1^2 z3^2 w1*z3^2 w1^2*z3 w1^2*z3^2];
assert(abs(coeff*PP) < 1e-9, 'PP polynomial (4) non-zero', coeff*PP);

% got everything.  copy D and M using notation of supplement to paper.
d0 = DD(1); d1 = DD(2); d2 = DD(3); d3 = DD(4); d4 = DD(5); 
m0 = MM(1); m1 = MM(2); m2 = MM(3); m3 = MM(4); m4 = MM(5); 

%syms w1 z3 d0 d1 d2 d3 d4 m0 m1 m2 m3 m4 %uncomment to double-check symbolically
U11 = d0 + d1*w1^2; U12 = d4*w1; U13 = d2 + d3*w1^2;
U31 = m0 + m2*z3^2; U32 = m4*z3; U33 = m1 + m3*z3^2;

%---------from paper supplement-----
Q00 = (d0*m1 - d2*m0)^2;
Q40 = (d1*m1 - d3*m0)^2;
Q20 = 2.0 * (d1 * m1 - d3 * m0) * (d0 * m1 - d2 * m0) + d4*d4*m0*m1;
Q11 = - d4 * m4 * (d0 * m1 + d2 * m0);
Q31 = - d4 * m4 * (d1 * m1 + d3 * m0);
temp1 = m4 * m4 - 2.0 * m0 * m3 - 2.0 * m1 * m2;
Q02 = 2.0 * d0 * d0 * m1 * m3 + 2.0 * d2 * d2 * m0 * m2 + d0 * d2 * temp1; 
Q42 = 2.0 * d1 * d1 * m1 * m3 + 2.0 * d3 * d3 * m0 * m2 + d1 * d3 * temp1; 
Q22 = 4.0 * d0 * d1 * m1 * m3 + 4.0 * d2 * d3 * m0 * m2 + d4 * d4 * (m0 * m3 + m1 * m2) + (d0 * d3 + d1 * d2) * temp1;
Q13 = - d4 * m4 * (d0 * m3 + d2 * m2);
Q33 = - d4 * m4 * (d1 * m3 + d3 * m2);
Q04 = (d0*m3 - d2*m2)^2 ;
Q44 = (d1*m3 - d3*m2)^2;
Q24 = 2.0 * (d1 * m3 - d3 * m2) * (d0 * m3 - d2 * m2) + d4*d4*m2*m3;

% aa = det([U11 U12 U13 0; 0 U11 U12 U13; U31 U32 U33 0; 0 U31 U32 U33]); % uncomment to check symbolicaly
% bb = Q00+Q20*w1^2+Q40*w1^4+Q11*w1*z3+Q31*w1^3*z3+Q33*w1^3*z3^3+Q02*z3^2+Q22*w1^2*z3^2+Q42*w1^4*z3^2+Q13*w1*z3^3+Q04*z3^4+Q24*w1^2*z3^4+Q44*w1^4*z3^4;
% expand(aa-bb)
assert(abs(U11+U12*z1+U13*z1^2)<1e-6,'U1 wrong',U11+U12*z1+U13*z1^2); %U's for checking/symbolic only...
assert(abs(U31+U32*z1+U33*z1^2)<1e-6,'U3 wrong',U31+U32*z1+U33*z1^2);
assert(abs(det([U11 U12 U13 0; 0 U11 U12 U13; U31 U32 U33 0; 0 U31 U32 U33])) < 1e-6, 'Det U wrong', ...
        det([U11 U12 U13 0; 0 U11 U12 U13; U31 U32 U33 0; 0 U31 U32 U33]));

assert(abs(Q00+Q20*w1^2+Q40*w1^4+Q11*w1*z3+Q31*w1^3*z3+Q33*w1^3*z3^3+Q02*z3^2+Q22*w1^2*z3^2+Q42*w1^4*z3^2+Q13*w1*z3^3+Q04*z3^4+Q24*w1^2*z3^4+Q44*w1^4*z3^4)<1e-6, '(D11)~=0',...
          [Q00+Q20*w1^2+Q40*w1^4+Q11*w1*z3+Q31*w1^3*z3+Q33*w1^3*z3^3+Q02*z3^2+Q22*w1^2*z3^2+Q42*w1^4*z3^2+Q13*w1*z3^3+Q04*z3^4+Q24*w1^2*z3^4+Q44*w1^4*z3^4]);

% set up A and B polynomials in W1, as in paper
a0 = [Q40 0 Q20 0 Q00];
a1 = [0 Q31 0 Q11 0];
a2 = [Q42 0 Q22 0 Q02];
a3 = [0 Q33 0 Q13 0];
a4 = [Q44 0 Q24 0 Q04];
b0 = PP([5 2 1])'; % [P20 P10 P00];
b1 = PP([8 4 3])'; % [P21 P11 P01];
b2 = PP([9 7 6])'; % [P22 P12 P02];

b0b0 = conv(b0,b0);
b1b1 = conv(b1,b1);
b2b2 = conv(b2,b2);
b0b1 = conv(b0,b1);
b1b2 = conv(b1,b2);
b2b0 = conv(b2,b0);
loop = conv(conv(a0,a0), conv(b2b2,b2b2));         % A0^2 B2^4 term 
loop = loop + conv(conv(a0,a4), conv(b1b1,b1b1));  % A0 A4 B1^4 
loop = loop - 2*conv(conv(a0,a2),conv(b2b0,b2b2)); % -2 A0 A2 B0 B2^3 
loop = loop - conv(conv(a0,a1), conv(b1b2,b2b2));  % -A0 A1 B1 B2^3 
loop = loop + conv(conv(a0,a2), conv(b1b1,b2b2));  % A0 A2 B1^2 B2^2 
loop = loop + 3*conv(conv(a0,a3), conv(b0b1,b2b2));% 3 A0 A3 B0 B1 B2^2 
loop = loop + 2*conv(conv(a0,a4), conv(b0b0,b2b2));% 2 A0 A4 B0^2 B2^2 
loop = loop - conv(conv(a0,a3), conv(b1b1,b1b2));  % -A0 A3 B1^3 B2 
loop = loop - 4*conv(conv(a0,a4), conv(b1b1,b2b0));% -4 A0 A4 B0 B1^2 B2 
loop = loop + conv(conv(a1,a1), conv(b2b0,b2b2));  % A1^2 B0 B2^3 
loop = loop - conv(conv(a1,a2), conv(b0b1,b2b2));  % -A1 A2 B0 B1 B2^2 
loop = loop - conv(conv(a1,a4), conv(b0b1,b1b1));  % -A1 A4 B0 B1^3 
loop = loop + conv(conv(a1,a3), conv(b2b0,b1b1));  % A1 A3 B0 B1^2 B2 
loop = loop + 3*conv(conv(a1,a4), conv(b0b0,b1b2));% 3 A1 A4 B0^2 B1 B2 
loop = loop - 2*conv(conv(a1,a3), conv(b0b0,b2b2));% -2 A1 A3 B0^2 B2^2 
loop = loop + conv(conv(a2,a2), conv(b0b0,b2b2));  % A2^2 B0^2 B2^2 
loop = loop + conv(conv(a2,a4), conv(b0b0,b1b1));  % A2 A4 B0^2 B1^2 
loop = loop - conv(conv(a2,a3), conv(b0b0,b1b2));  % -A2 A3 B0^2 B1 B2 
loop = loop - 2*conv(conv(a2,a4), conv(b0b0,b2b0));% -2 A2 A4 B0^3 B2 
loop = loop - conv(conv(a3,a4), conv(b0b0,b0b1));  % -A3 A4 B0^3 B1 
loop = loop + conv(conv(a3,a3), conv(b0b0,b2b0));  % A3^2 B0^3 B2 
loop = loop + conv(conv(a4,a4), conv(b0b0,b0b0));  % A4^2 B0^4 

assert(abs(polyval(loop, w1))<1e-8, 'w1 is not a root of R', polyval(loop, w1));
rts = roots(loop);
W1 = rts(abs(imag(rts)) < 1e-9);
%wdiff = w1-W1

% compute Z3 from W1
A0 = polyval(a0,W1); A1 = polyval(a1,W1); A2 = polyval(a2,W1); A3 = polyval(a3,W1); A4 = polyval(a4,W1);
B0 = polyval(b0,W1); B1 = polyval(b1,W1); B2 = polyval(b2,W1);
% I double-checked that two numerical evaluations of the polynomial give nearly zeros.
K1 = A3.*B2-A4.*B1;
K3 = (A2.*B2-A4.*B0).*B2-K1.*B1;
Z3 = (K3.*B0-A0.*B2.^3)./((A1.*B2.^2-K1.*B0).*B2-K3.*B1);
%z3diff = z3-Z3 

% compute Z1 from W1 and Z3
U11 = d0 + d1.*W1.^2; U12 = d4.*W1; U13 = d2 + d3.*W1.^2;
U31 = m0 + m2.*Z3.^2; U32 = m4.*Z3; U33 = m1 + m3.*Z3.^2;
Z1 = (U31.*U13-U11.*U33)./(U12.*U33 - U13.*U32);
% z1diff = z1-Z1

fprintf('We started with w1=%f, z1=%f, z3=%f, and now have %d solutions:\n', w1, z1, z3, size(W1,1));
disp([W1 Z1 Z3]);

disp('Now we plug in W1, Z1, Z3 to verify that their polynomials have zeros:');
Rquality = [polyval(loop, W1) (A0.^2.*B2.^4 +A0.*A4.*B1.^4 -2.*A0.*A2.*B0.*B2.^3 -A0.*A1.*B1.*B2.^3 +A0.*A2.*B1.^2.*B2.^2 +3.*A0.*A3.*B0.*B1.*B2.^2 +2.*A0.*A4.*B0.^2.*B2.^2 -A0.*A3.*B1.^3.*B2 -4.*A0.*A4.*B0.*B1.^2.*B2 +A1.^2.*B0.*B2.^3 -A1.*A2.*B0.*B1.*B2.^2 -A1.*A4.*B0.*B1.^3 +A1.*A3.*B0.*B1.^2.*B2 +3.*A1.*A4.*B0.^2.*B1.*B2 -2.*A1.*A3.*B0.^2.*B2.^2 +A2.^2.*B0.^2.*B2.^2 +A2.*A4.*B0.^2.*B1.^2 -A2.*A3.*B0.^2.*B1.*B2 -2.*A2.*A4.*B0.^3.*B2 -A3.*A4.*B0.^3.*B1 +A3.^2.*B0.^3.*B2 +A4.^2.*B0.^4)]
DPMqualities = [[ones(size(W1)) W1.^2 Z1.^2 W1.^2.*Z1.^2 W1.*Z1]*DD ...
                [ones(size(W1)) W1 Z3 W1.*Z3 W1.^2 Z3.^2 W1.*Z3.^2 W1.^2.*Z3 W1.^2.*Z3.^2]*PP ...
                [ones(size(W1)) Z1.^2 Z3.^2 Z1.^2.*Z3.^2 Z1.*Z3]*MM]


%fprintf('angles were om1=%f ze1=%f ze3=%f, now:\n', sa([om1, ze1, ze3])/degrees);
%angles = 2*atan([W1 Z1 Z3])/degrees

result = zeros(9,3,size(W1,1)); % make coordinates from angles
domega = 2*atan(W1) - sa(om1); % residual rotation angles for Omega1
omega1 = 2*atan(W1);
zeta1 = 2*atan(Z1);
zeta3 = 2*atan(Z3);

% We undo the rotation of original peptide planes to prepare to build output coords
pS1 = rotby(v(s1,:), V(q1,:), -sa(ze1)); 
pB1 = rotby(v(pb1,:), V(q1,:), -sa(ze1)); 
pB3 = rotby(v(pb3,:), V(q3,:), -sa(ze3)); 
pP3 = rotby(v(p3,:), V(q3,:), -sa(ze3)); 
assert(abs(dot(pS1,cross(V(r,:), V(q1,:)))) < 1e-9, ' failed to rotate peptide pS1 to q1r plane');
assert(abs(dot(pB1,cross(-V(r,:), V(q1,:)))) < 1e-9, ' failed to rotate peptide pB1 to q1r plane');
assert(abs(dot(pB3,cross(-V(r,:), V(q3,:)))) < 1e-9, ' failed to rotate peptide pB3 to q3r plane');
assert(abs(dot(pP3,cross(V(r,:), V(q3,:)))) < 1e-9, ' failed to rotate peptide pP3 to q3r plane');

fprintf('Building coords');
for i = 1:size(W1,1) 
    fprintf(', %d', i);
    result([n1 ca1 ca3 c3],:,i) = chain([n1 ca1 ca3 c3],:); % copy stationary atoms
% We use the residual omega rotation to rotate q1 to a new position Q1
    Q1 = rotby(v(q1,:), V(r,:), domega(i)); 
    result(ca2,:,i) = chain(ca1,:) + Q1;
    Q3 = Q1 - v(r,:); % need vector Q3 as basis for peptide rotation
    assert(abs(norm(Q1)-norm(v(q1,:))) < 1e-9, 'Q1 changed length!', [norm(Q1) norm(v(q1,:))]);
    assert(abs(norm(Q3)-norm(v(q3,:))) < 1e-9, 'Q3 changed length!', [norm(Q3) norm(v(q3,:))]);
    Q1 = Q1/norm(Q1); Q3 = Q3/norm(Q3);
   
    assert(abs(dihedral(Q1, V(r,:), V(p1,:))-omega1(i))<1e-6, ...
        'wrong dihedral for Q1rp1', [dihedral(Q1, V(r,:), V(p1,:)) omega1(i)]);
    assert(abs(dihedral(Q3, V(r,:), V(p1,:))-omega1(i))<1e-6, ...
        'wrong dihedral for Q3rp1', [dihedral(Q3, V(r,:), V(p1,:)) omega1(i)]);

% Rotate peptide plane around Q1: zeta1 is angle from S1Q1 to rQ1
    
    result(c1,:,i) = chain(ca1,:) + rotby(rotby(pS1, V(r,:), domega(i)), Q1, zeta1(i));
    result(n2,:,i) = chain(ca1,:) + rotby(rotby(pB1, V(r,:), domega(i)), Q1, zeta1(i));
    result(c2,:,i) = chain(ca3,:) + rotby(rotby(pB3, V(r,:), domega(i)), Q3, zeta3(i));
    result(n3,:,i) = chain(ca3,:) + rotby(rotby(pP3, V(r,:), domega(i)), Q3, zeta3(i));

% Check invariants of the solution
    assert(abs(dihedral(result(c1,:,i)-chain(ca1,:), Q1, V(r,:))-zeta1(i))<1e-6, ...
        'wrong dihedral for pS1', [i dihedral(result(c1,:,i)-chain(ca1,:), Q1, V(r,:)) zeta1(i)]);
    assert(abs(dihedral(result(n2,:,i)-chain(ca1,:), Q1, -V(r,:))-zeta1(i))<1e-6, ...
        'wrong dihedral for pB1', [i dihedral(result(n2,:,i)-chain(ca1,:), Q1, -V(r,:)) zeta1(i)]);
    assert(abs(dihedral(result(c2,:,i)-chain(ca3,:), Q3, -V(r,:))-zeta3(i))<1e-6, ...
        'wrong dihedral for pB3', [i dihedral(result(c2,:,i)-chain(ca3,:), Q3, -V(r,:)) zeta3(i)]);
    assert(abs(dihedral(result(n3,:,i)-chain(ca3,:), Q3, V(r,:))-zeta3(i))<1e-6, ...
        'wrong dihedral for pP3', [i dihedral(result(n3,:,i)-chain(ca3,:), Q3, V(r,:)) zeta3(i)]);
    
    vv = diff(result(:, :,i)); vv2 = diff(chain(:,:)); lv = sqrt(dot(vv2,vv2,2));
    assert(all(abs(sqrt(dot(vv,vv,2))-lv)<1e-6), 'Bond lengths change', [sqrt(dot(vv2,vv2,2)) lv]);
    
    VV = vv./repmat(sqrt(dot(vv,vv,2)),1,3); VV2 = vv2./lv(:,[1 1 1]);
    ba = acos([dot(VV2(1:(size(VV,1)-1),:), VV2(2:end,:),2) dot(VV(1:(size(VV,1)-1),:), VV(2:end,:),2)]);
    assert(all(abs(diff(ba,1,2))<1e-6), 'Bond angles change', ba);
    
    vv = result([ca1 ca2 n3, ca2  ca2, ca3, c1  ca2 ca3, n2 c2], :,i) ...
        -result([n1  n2 ca3, ca1  ca3, ca1, ca1 c2  c3,  ca1 ca3], :,i);
    assert(all(abs(lengths - sqrt(dot(vv,vv,2)))<1e-6),'Lengths change', [lengths sqrt(dot(vv,vv,2))]);

    VV = vv./repmat(sqrt(dot(vv,vv,2)),1,3);
    vang = acos(dot(VV([p1 p2 s3, q1 q1 q3, q1 q3 p3, p1 p2 p3, q1 q3],:), ...
                    VV([r  q1 r,  r  q3 r,  s1 s2 q3, s1 s2 s3, p1 s3],:),2));
    assert(all(abs(vang(1:12)-ang(1:12))<1e-6), 'Vector angles change', [ang(1:12) vang(1:12)]);
           
end
fprintf(', & done\n');

function assert(flag, string, val)
if (~flag)
    fprintf('ASSERT: %s\n', string);
    if nargin > 2
        disp(val);
    end
end

function result = dihedral(p,r,q)
% dihedral(p,r,q) returns angle in radians from plane pr to plane qr,
% measured ccw around r. The p,r,q may be lists of vectors of the same length

y = cross(r,p,2); y = y./repmat(sqrt(dot(y,y,2)),1,3);
x = cross(y,r,2); x = x./repmat(sqrt(dot(x,x,2)),1,3);
result = atan2(dot(q,y,2), dot(q,x,2));

function result = rotby(p, r, theta)
% rotby(p, r, theta) returns the vector p rotated around unit vector r by theta.
% for now, assume that p and r are not colinear

assert(abs(1-norm(r))<1e-9, 'r must be unit vector in rotby', r);
y = cross(r,p); y = y/norm(y);
x = cross(y,r); x = x/norm(x);
pr = dot(p,r); px = dot(p,x);
%assert((abs(dot(p,y)) < 1e-9), 'p not orthog to y', dot(p,y));
result = px * (cos(theta)*x - sin(theta)*y) + pr * r;
%assert(abs(dot(p,r)-dot(result,r))<1e-9, 'changed inner angle in rotby', [dot(p,r) dot(result,r)]);

